package edu.gb.mummec.matrix;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

class Matrix {
    
    private double[][] theMatrix;
    private Matrix() { }
    
    public Matrix(String filename) throws FileNotFoundException, IOException, MatrixException {
        List<double[]> filas=new ArrayList<>();
        
        BufferedReader br=new BufferedReader(new FileReader(filename));
        String linea;
        while ((linea=br.readLine())!=null) {
            if (linea.trim().length()==0) continue;
            String[] campos=linea.split("\\s+");
            
            if (filas.size()>0 && campos.length!=filas.get(0).length)
                throw new MatrixException("La matriz no es rectangular (la fila "+filas.size()+" es diferente a las anteriores)");
            
            
            double[] valores=new double[campos.length];
            for (int columna=0; columna<campos.length; columna++) {
                try {
                    valores[columna]=Double.parseDouble(campos[columna]);
                } catch(NumberFormatException ex) {
                    throw new MatrixException("El elemento ("+filas.size()+","+columna+") no tiene formato numérico ("+campos[columna]+")");
                }
            }
            filas.add(valores);
        }
        if (filas.size()==0) throw new MatrixException("El fichero "+filename+" no contiene una matriz");
        theMatrix=filas.toArray(new double[0][0]);
    }
    
    
     public static Matrix traspose(Matrix M){
        Matrix newMatrix= new Matrix();
        newMatrix.theMatrix= new double[M.theMatrix[0].length][M.theMatrix.length];
        for (int fila=0; fila<M.theMatrix.length; fila++) 
            for (int columna=0; columna<M.theMatrix[0].length; columna++)
                newMatrix.theMatrix[columna][fila]=M.theMatrix[fila][columna];
        return newMatrix;
}
    
    
    public void traspose(){
         theMatrix=traspose(this).theMatrix;
    }
    

    @Override
    public String toString() {
        StringBuffer sb=new StringBuffer();
        for (int fila=0; fila<theMatrix.length; fila++){
            sb.append("[");
            for (int columna=0; columna<theMatrix[fila].length-1; columna++)
                sb.append(theMatrix[fila][columna]).append(",\t");
            sb.append(theMatrix[fila][theMatrix[fila].length-1]).append("]\n");
        }
        return sb.toString();
    }

    
    
}